/**
 * eobjects.org MetaModel
 * Copyright (C) 2010 eobjects.org
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */
package org.eobjects.metamodel.util;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Represents a lazy loaded reference.
 * 
 * @author Kasper Sørensen
 * 
 * @param <E>
 */
public abstract class LazyRef<E> implements Ref<E> {

	private final AtomicBoolean _fetched = new AtomicBoolean(false);
	private E _object;

	@Override
	public final E get() {
		if (!_fetched.get()) {
			synchronized (_fetched) {
				if (!_fetched.get()) {
					_object = fetch();
					_fetched.set(true);
				}
			}
		}
		return _object;
	}

	protected abstract E fetch();

	/**
	 * Gets whether the lazy loaded reference has been loaded or not.
	 * 
	 * @return a boolean indicating whether or not the reference has been loaded
	 *         or not
	 */
	public boolean isFetched() {
		return _fetched.get();
	}

	/**
	 * Requests an asynchronous load of the lazy reference. If not already
	 * loaded, this will cause another thread to load the reference, typically
	 * to make it immediately available for later evaluation.
	 */
	public void requestLoad() {
		if (!isFetched()) {
			ExecutorService executorService = SharedExecutorService.get();
			executorService.submit(new Runnable() {
				@Override
				public void run() {
					get();
				}
			});
		}
	}

}
